﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace LinqSchool
{
    class Program
    {
        private static readonly IEnumerable<Teacher> Teachers = new List<Teacher>
        {
            new Teacher {Id = 1, Name = "Anders", Salary = 1000},
            new Teacher {Id = 2, Name = "John", Salary = 1200},
            new Teacher {Id = 4, Name="Dennis", Salary = 1100},
            new Teacher {Id = 3, Name="Brian", Salary = 1000}
        };

        private static readonly IEnumerable<Course> Courses = new List<Course>
        {
            new Course {Name = "Programming", Credits = 15, TeacherId = 1},
            new Course {Name="Network", Credits = 10, TeacherId = 2},
            new Course {Name="Systems Development", Credits = 5, TeacherId = 4},
            new Course {Name="Systems Development 2", Credits = 10, TeacherId = 4}
        };

        static void Main()
        {
            AllTeachers();
            //HighSalaries();
            //TeachersByName();
            //JoiningTeacherCourse();
            //GroupByExample();
        }



        private static void AllTeachers()
        {
            var allTeachers = from teacher in Teachers
                              select teacher;
            Console.WriteLine("All teachers, Query syntax: ");
            Console.WriteLine(String.Join("\n", allTeachers));
            var allTeachers2 = Teachers.Select(teacher => teacher);
            // Select(...) is not necessary in this case, since we want ALL the teachers
            Console.WriteLine("All teachers, method syntax: ");
            Console.WriteLine(String.Join("\n", allTeachers2));
        }

        private static void HighSalaries()
        {
            int limit = 1100;
            var highSalaries = from teacher in Teachers
                               where teacher.Salary >= limit
                               select teacher;
            limit = 400;
            Console.WriteLine("Teachers with salary >= {0}", limit);
            Console.WriteLine(String.Join("\n", highSalaries));

            var highSalaries2 = Teachers.Where(teacher => teacher.Salary >= 1100);
            Console.WriteLine("Teachers with high salaries: ");
            Console.WriteLine(String.Join("\n", highSalaries2));
        }

        private static void TeachersByName()
        {
            var teachersByName = from teacher in Teachers
                                 orderby teacher.Name
                                 select teacher;
            Console.WriteLine("Teachers sorted by name, query syntax");
            Console.WriteLine(String.Join("\n", teachersByName));

            var teachersByName2 = Teachers.OrderBy(teacher => teacher.Name);
            Console.WriteLine("Teachers sorted by name, method syntax");
            Console.WriteLine(String.Join("\n", teachersByName2));
        }

        private static void JoiningTeacherCourse()
        {
            var res = from course in Courses
                      join teacher in Teachers on course.TeacherId equals teacher.Id
                      select new { CourseName = course.Name, TeacherName = teacher.Name };
            Console.WriteLine("Teacher join course, query syntax");
            Console.WriteLine(String.Join("\n", res));

            var res2 = Courses.Join(Teachers,
                course => course.TeacherId,
                teacher => teacher.Id,
                (course, teacher) => new { Course = course.Name, Teacher = teacher.Name }
                );
            Console.WriteLine("Teacher join course, method syntax");
            Console.WriteLine(String.Join("\n", res2));
        }

        private static void GroupByExample()
        {
            var res = from course in Courses
                      group course by course.TeacherId
                          into gr
                          select new { Teacher = gr.Key, Count = gr.Count() };
            Console.WriteLine("Courses group by teacherId, query syntax");
            Console.WriteLine(String.Join("\n", res));

            var res2 = Courses.GroupBy(course =>
                course.TeacherId).
                Select(gr => new { Teacher = gr.Key, HowMany = gr.Count() });
            Console.WriteLine("Courses group by teacherId, method syntax");
            Console.WriteLine(String.Join("\n", res2));
        }
    }
}
